/*
 * bahtinov.scad -- bahtinov/sandor/cuzdi mask generator
 *
 * (c) 2014 Prof Dr Andreas Mueller
 */
include <parameters.scad>

// no user modifiable parameters below this line
landwidth = slotseparation - slotwidth;
width = 6;
$fn = 60;
gap = 0.1;
holewallthickness = 1.5;
wallthickness = 2;

// create the slots pattern, this can later be used to remove the slots from the
// body of the mask
module slots() {
	union() {
		intersection() {
			union() {
				for (offset = [slotseparation/2: slotseparation: radius]) {
					translate([-radius - width/2,  offset - (slotwidth / 2), -1])
						cube([radius, slotwidth, 10]);
					translate([-radius - width/2, -offset - (slotwidth / 2), -1])
						cube([radius, slotwidth, 10]);
				}
			}
			difference() {
				cylinder(r = radius - 4, h = 10, center = true);
				cylinder(r = hole + 4, h = 12, center = true);
			}
		}
	
		intersection() {
			union() {
				intersection() {
					translate([width/2, width/2, 0])
						cube([radius, radius, 20]);
					for (offset = [-radius * sin(angle): slotseparation: radius]) {
						rotate([0, 0,  angle]) translate([5,  offset, -1])
							cube([2 * radius, slotwidth, 10], center = true);
					}
				}
				intersection() {
					translate([width/2, -radius - width/2, 0])
						cube([radius, radius, 20]);
					for (offset = [-radius * sin(angle): slotseparation: radius]) {
						rotate([0, 0, -angle]) translate([5, -offset, -1])
							cube([2 * radius, slotwidth, 10], center = true);
					}
				}
			}
			difference() {
				cylinder(r = radius - 4, h = 10, center = true);
				cylinder(r = hole + 4, h = 12, center = true);
			}
		}
	}
}

// create the body of the mask
module body() {
	difference() {
		// outer part of the body
		cylinder(r = radius, h = height);
		union() {
			// inner ring of the, leaves a 2mm wide wall on the outside
			// and a 1.5mm thick wall around the hole
			translate([0, 0, thickness])
				difference() {
					cylinder(r = radius - wallthickness, h = height, $fn = 120);
					cylinder(r = hole + holewallthickness, h = height, $fn = 120);
				}
			translate([0, 0, thickness + holeheight]) cylinder(r = hole + 10, h = height);
		}
	}
}

// for easiear removal from the telescope, knobs should be added
// to the mask. The knobholes function creates the holes for those knobs
module	knobholes(offset) {
	if ((hole == 0) && (radius < 80)) {
		translate([0, 0, (height + 1) / 2])
			cylinder(r = 2, h = height + 1, center = true);
	} else {
		translate([0, offset, 0])
			cylinder(r = 2, h = 10, center = true);
		translate([0, -offset, 0])
			cylinder(r = 2, h = 10, center = true);
	}
}

// the knob function creates a single knob
module knob() {
	union() {
		translate([0, 0, -5])
			cylinder(r = 3, h = 10, center = true);
		cylinder(r = 2, h = 2 * thickness, center = true);
	}
}

// the showknobs function places two knobs exactly in the position
// where the holes are, this can be used in case the mask is produced
// in powder printer, which does not have the same restrictions as a
// FDM printer
module showknobs() {
	if ((hole == 0) && (radius < 80)) {
		knob();
	} else {
		translate([0, radius - 10, 0])
			knob();
		translate([0, -radius + 10, 0])
			knob();
	}
}

// produce a full bahtinov mask in one piece
module fullmask() {
	difference() {
		body();
		union() {
			slots();
			cylinder(r = hole, h = height + thickness);
			knobholes(radius - 10);
		}
	}
}

// create a cap or ring that can be used to glue the sectors of a bahtinov
// mask together, if it cannot be printed in a single piece
module ring() {
	difference() {
		cylinder(r = hole + 2 * holewallthickness + gap, h = holeheight + thickness * fillhole);
		translate([0, 0, - 0.01])
			cylinder(r = hole + holewallthickness + gap, h = holeheight + 0.02);
	}
}

// create the flap that is needed to glue the outer frame portions together
module flap1(internalradius, externalradius, offset) {
	intersection() {
		difference() {
			translate([0, 0, offset]) cylinder(r = externalradius, h = height - offset, $fn = 120);
			translate([0, 0, offset]) cylinder(r = internalradius, h = height - offset, $fn = 120);
		}
		intersection() {
			rotate([0, 0, -15])
				translate([0, 0, -0.5])
					cube([externalradius + 1, externalradius, height + thickness]);
			rotate([0, 0, +15]) translate([0, -externalradius])
				translate([0, 0, -0.5])
					cube([externalradius + 1, externalradius, height + thickness]);
		}
	}
}

// place the flaps, on the onside or the outside. The flap has a thicknes 
// of 1mm
module flap(inside) {
	if (inside == 0) {
		flap1(radius, radius + 1, 0);
	} else {
		flap1(radius - wallthickness - 1, radius - wallthickness, 1);
	}
}

// parameters for the connecting blocks in the multipart bahtinov mask
v = 10;
epsilon = 0.5;

// create the bonding rectangles along the seams
module bonding(q, e) {
	for (r = [hole + 3 + v/2 + e * v: 2 * v: radius - 4 - v/2]) {
		if ((landwidth < width) && (q == 2)) {
			rotate([0, 0, q * 90]) translate([r, 0, 1.5])
				cube([v - epsilon, landwidth, 1], center = true);
		} else {
			rotate([0, 0, q * 90]) translate([r, 0, 1.5])
				cube([v - epsilon, width, 1], center = true);
		}
	}
}

// create a quadrant of the mask. The second argument indicates whether
// the flaps should be placedon the inside or the outside of the rim.
// This choice may depend on whether the mask is placed over the the
// rim of the telescope, or within (depending on the dew shield, e.g.).
module quadrant(q, inside) {
	union() {
		if ((q == 0) || (q == 4)) {
			intersection() {
				translate([0, 0, -thickness]) cube([radius, radius, height + 2 * thickness]);
				fullmask();
			}
		}
		if (q == 1) {
			intersection() {
				translate([-radius, 0, -thickness]) cube([radius, radius, height + 2 * thickness]);
				fullmask();
			}
		}
		if (q == 2) {
			intersection() {
				translate([-radius, -radius, -thickness]) cube([radius, radius, height + 2 * thickness]);
				fullmask();
			}
		}
		if (q == 3) {
			intersection() {
				translate([0, -radius, -thickness]) cube([radius, radius, height + 2 * thickness]);
				fullmask();
			}
		}
		rotate([0, 0, 90 * q]) flap(inside);
		bonding(q, 0);
		bonding(q + 1, 1);
	}
}

// produce all sectors of the mask
module allsectors1(offset, inside, explode) {
	translate([0, 0, (1 - explode) * height/2]) rotate([180 * (1 - explode), 0, 0]) ring();
	translate([ offset * explode,  offset * explode, ]) quadrant(0, inside);
	translate([-offset * explode,  offset * explode, ]) quadrant(1, inside);
	translate([-offset * explode, -offset * explode, ]) quadrant(2, inside);
	translate([ offset * explode, -offset * explode, ]) quadrant(3, inside);
}

module allsectors(randinside, explode) {
	allsectors1(5 + tan(15) * radius / 2, randinside, explode);
}


// allsectors(1, 1);
// fullmask();
// showknobs();
// ring();
